/*
 * (C) 2007, Tom Wheeler.  All rights reserved.
 */

package com.tomwheeler.demo.jcr.worldinfo;

import java.io.IOException;
import java.util.GregorianCalendar;
import java.util.List;
import javax.jcr.Session;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author twheeler
 */
public class CountryManagerTest {
    
    private static CountryManager manager;
    private static Session session;
    
    @BeforeClass
    public static void testBeforeTasks() throws IOException {
        RepoManager.deleteRepository();
        
        session = RepoManager.getSession();
        manager = new CountryManager(session);

        addThree(manager); // indirectly tests add
    }
    
    @AfterClass
    public static void testAfterTasks() throws IOException {
        session.logout();
        RepoManager.deleteRepository();
    }

    @Test
    public void update() {        
        Country brazil = new Country("Brazil", "South America");
        brazil.setCurrency("New Real");
        brazil.setPrincipalLanguage("New Portuguese");
        brazil.setPopulation(500);
        brazil.setDateFormed(new GregorianCalendar(1920, 5, 20));
        brazil.setTld(".nu");
        manager.update(brazil);
        
        Country newBrazil = manager.findByTld(".nu");
        assertEquals("New Portuguese", newBrazil.getPrincipalLanguage());
        assertEquals("New Real", newBrazil.getCurrency());
        assertEquals(500L, newBrazil.getPopulation());
    }

    @Test(expected=DataAccessException.class)
    public void delete() {
        Country canada = manager.findByTld(".ca");
        assertNotNull(canada);
        assertEquals("English", canada.getPrincipalLanguage());
        
        manager.delete(canada);
        manager.findByTld(".ca"); // should throw DataAccessException
    }

    @Test
    public void findByTld() {
        Country argentina = manager.findByTld(".ar");
        assertNotNull(argentina);
        assertEquals("Argentina", argentina.getName());
        assertEquals("South America", argentina.getContinent());
        assertEquals("Peso", argentina.getCurrency());
        assertEquals("Spanish", argentina.getPrincipalLanguage());
        assertEquals(40000000L, argentina.getPopulation());
        assertEquals(".ar", argentina.getTld());
    }

    @Test
    public void getAllCountries() {
        boolean sawArgentina = false;
        boolean sawBrazil = false;
        boolean sawCanada = false;

        List<Country> countries = manager.getAllCountries();
        for (Country country : countries) {
            if ("Argentina".equals(country.getName())) {
                sawArgentina = true;
            } else if ("Brazil".equals(country.getName())) {
                sawBrazil = true;
            } else if ("Canada".equals(country.getName())) {
                sawCanada = true;
            } else {
                fail("Found unknown country '" + country.getName() + "'");
            }
        }
        
        assertTrue(sawArgentina);
        assertTrue(sawBrazil);
        assertTrue(sawCanada);
    }
    
    private static void addThree(CountryManager manager) {
        Country argentina = new Country("Argentina", "South America");
        argentina.setCurrency("Peso");
        argentina.setDateFormed(new GregorianCalendar(1816, 6, 16));
        argentina.setPrincipalLanguage("Spanish");
        argentina.setPopulation(40000000);
        argentina.setTld(".ar");
        manager.add(argentina);

        Country brazil = new Country("Brazil", "South America");
        brazil.setCurrency("Real");
        brazil.setDateFormed(new GregorianCalendar(1822, 8, 7));
        brazil.setPrincipalLanguage("Portuguese");
        brazil.setPopulation(190000000);
        brazil.setTld(".br");
        manager.add(brazil);
        
        Country canada = new Country("Canada", "North America");
        canada.setCurrency("Dollar");
        canada.setDateFormed(new GregorianCalendar(1867, 6, 1));
        canada.setPrincipalLanguage("English");
        canada.setPopulation(190000000);
        canada.setTld(".ca");
        manager.add(canada);
    }
}
